import matplotlib.pyplot as plt
import matplotlib.dates as mdates

close_plots: bool = True

def create_figure(name: str, return_ax: bool = True):
    """Helper function to create a figure with a given name that returns the figure and axis
    """
    if close_plots:
        plt.close(name)
    fig = plt.figure(name)
    fig.suptitle = name

    if return_ax:
        return fig, fig.subplots()
    return fig

def set_ax_timeformat(ax, timeformat = "%d-%m-%Y %H:%M:%S", rotation = 10):
    if not hasattr(ax, '__iter__'):
        ax = [ax]
    
    for a in ax:
        a.xaxis.set_major_formatter(mdates.DateFormatter(timeformat))
        a.xaxis.set_major_locator(mdates.AutoDateLocator())
        a.tick_params(axis='x', rotation=rotation)

# 
def clear_memory():
    import gc
    gc.collect()
    
    # Trim cache of gcc libc library
    try:
        import ctypes
        libc = ctypes.CDLL("libc.so.6") # clearing cache 
        libc.malloc_trim(0)
    except:
        pass